<?php

namespace Drupal\tmgmt_xillio;

use Drupal\tmgmt\TranslatorPluginUiBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\tmgmt\JobInterface;

/**
 * Xillio translator UI.
 */
class XillioTranslatorUi extends TranslatorPluginUiBase {

  /**
   * The api connector.
   *
   * @var \Drupal\tmgmt_xillio\TapiccConnector
   */
  private $apiConnector;

  /**
   * XillioTranslatorUI constructor.
   *
   * @param array $configuration
   *   The plugin config.
   * @param string $plugin_id
   *   The plugin id
   * @param mixed $plugin_definition
   *   The plugin definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->apiConnector = \Drupal::service('tmgmt_xilio.tapicc_connector');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    $form_state_settings = $form_state->getUserInput()['settings'];

    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $translator = $form_state->getFormObject()->getEntity();
    $settings = $translator->getSettings();

    $form['service_url'] = [
      '#type' => 'textfield',
      '#title' =>$this->t('xillio API url'),
      '#default_value' => $settings['service_url'],
      '#description' =>$this->t('The url of the translation instance without trailing slash.'),
      '#required' => TRUE,
      '#placeholder' => 'https://example.lochub.com',
    ];
    $form['username'] = [
      '#type' => 'textfield',
      '#title' =>$this->t('Xillio username'),
      '#default_value' => $settings['username'],
      '#required' => TRUE,
    ];

    $form['password'] = [
      '#type' => 'textfield',
      '#title' =>$this->t('Xillio password'),
      '#required' => TRUE,
      '#default_value' => $settings['password'],
    ];

    $form += parent::addConnectButton();

    // If connected, get projects.
    // Get settings from either the translator or form state.
    $available_settings = [];
    foreach (['username', 'password', 'service_url'] as $setting) {
      if (!empty($settings[$setting])) {
        $available_settings[$setting] = $settings[$setting];
      }
      elseif (!empty($form_state_settings[$setting])) {
        $available_settings[$setting] = $form_state_settings[$setting];
      }
    }

    if (count($available_settings) == 3 && $this->apiConnector->setServiceUrl($available_settings['service_url'])->ping()) {
      $projects = $this->apiConnector
        ->setServiceUrl($available_settings['service_url'])
        ->setAuthToken($available_settings['username'], $available_settings['password'])
        ->getProjects();

      $project_options = [];
      foreach ($projects as $project) {
        $project_options[$project['id']] = $project['name'];
      }

      $form['project'] = [
        '#type' => 'select',
        '#title' => $this->t('Project'),
        '#description' => $this->t('Select the project used for this provider'),
        '#options' => $project_options,
        '#default_value' => $translator->getSetting('project'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    if ($form_state->hasAnyErrors()) {
      return;
    }

    /** @var \Drupal\tmgmt\TranslatorInterface $translator */
    $settings = $form_state->getValue('settings');

    try {
      $result = $this->apiConnector
        ->setServiceUrl($settings['service_url'])
        ->getAuthToken($settings['username'], $settings['password'], TRUE);
    }
    catch (\Exception $e) {
      $form_state->setErrorByName('settings',$this->t($e->getMessage()));
    }

    if (!$result) {
      $form_state->setErrorByName('settings',$this->t('Connection to the service provider failed, please check url and credentials.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function checkoutSettingsForm(array $form, FormStateInterface $form_state, JobInterface $job) {
    $settings['due_at'] = [
      '#type' => 'datetime',
      '#title' =>$this->t('Due at'),
      '#description' =>$this->t('Enter the due date for those translations'),
      '#default_value' => $job->getSetting('due_at') ? $job->getSetting('due_at') : '',
      '#required' => TRUE,
    ];

    $settings['description'] = [
      '#type' => 'textarea',
      '#title' =>$this->t('Describe this job'),
      '#description' =>$this->t('Enter a description for his translation job'),
      '#default_value' => $job->getSetting('description') ? $job->getSetting('description') : '',
    ];

    return $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function checkoutInfo(JobInterface $job) {
    // If the job is finished, it's not possible to import translations anymore.
    if (!$job->isActive()) {
      return parent::checkoutInfo($job);
    }

    $form = [
      '#type' => 'fieldset',
      '#title' => t('Fetch translations'),
    ];

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Pull translations'),
      '#submit' => [
        [$this, 'pullTranslations'],
      ],
    );

    return $form;
  }

  /**
   * Calback for the pull translations button.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state interface.
   *
   * @throws \Drupal\tmgmt\TMGMTException
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function pullTranslations(array $form, FormStateInterface $form_state) {
    $messenger = \Drupal::messenger();
    /** @var \Drupal\tmgmt\JobInterface $job */
    $job = $form_state->getFormObject()->getEntity();
    $translator = $job->getTranslator();

    /** @var \Drupal\tmgmt_xillio\TapiccConnector $api_connector */
    $api_connector = \Drupal::service('tmgmt_xilio.tapicc_connector');
    $api_connector->configure($translator);

    try {
      /** @var \Drupal\tmgmt\RemoteMappingInterface $remote_mapping */
      foreach ($job->getRemoteMappings() as $remote_mapping) {
        $task_id = $remote_mapping->getRemoteIdentifier3();
        $task_status = $api_connector->checkTaksStatus($task_id);

        if (isset($task_status['status']) && in_array($task_status['status'], ['completed', 'completed-with-warnings'])) {
          $translator->getPlugin()->addTranslation($remote_mapping);
          $messenger->addMessage(t('Translation pulled successfully'));
        }
        elseif (isset($task_status['status']) && in_array($task_status['status'], ['cancelled', 'failed'])) {
          $job_item = $remote_mapping->getJobItem();
          $job_item->abortTranslation();
          $job->addMessage(t('Job item @item was aborted because the service marked is as @status', [
            "@item" => $job_item->label(),
            "@status" => $task_status['status'],
          ]));
        }
        else {
          $messenger->addMessage(t('Translations are not yet completed. Please retry later'));
        }
      }
    }
    catch (\Exception $e) {
      $job->addMessage(t('An error occurred when pulling translations: @error', [
        '@error' => $e->getMessage(),
      ]));
    }

  }

}
